
const DELICIOUSPOST_URL = "chrome://aedeliciouspost/";
const DELICIOUSPOST_HTTP_REALM = "Delicious Post";
const DELICIOUS_API_URL = "https://api.del.icio.us/v1/";

var delicious_post_stringbundle;
var oldpswd, nsLoginInfo;


function alertEx(aTitle, aMessage)
{
  var prmpt = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
    .getService(Components.interfaces.nsIPromptService);
  prmpt.alert(null, aTitle, aMessage);
}


function delicious_post_dialog_init() {
  // For login manager code that would be executed in Firefox 3 or newer
  if ("@mozilla.org/login-manager/loginInfo;1" in Components.classes) {
    nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1", Components.interfaces.nsILoginInfo, "init");
  }

  var username='';
  var password='';
	
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

  if (preferencesService.prefHasUserValue('extensions.aecreations.deliciouspost.username')) {
    var username=preferencesService.getCharPref('extensions.aecreations.deliciouspost.username');
    if (preferencesService.getBoolPref('extensions.aecreations.deliciouspost.use_password_manager')==true) {
      password = delicious_getPassword(username) || "";
      oldpswd = password;
      document.getElementById('delicious-post-dialog-password').setAttribute('value',password);
      document.getElementById('delicious-post-dialog-use_password_manager').setAttribute('checked',true);
    }
  }

  var description=window.arguments[1]; // this is passed to the dialog by the delicious_post_submit() function in context.js
  var url=window.arguments[0]; 
	
  // set the values in the dialog
  document.getElementById('delicious-post-dialog-username').setAttribute('value',username);
  document.getElementById('delicious-post-dialog-url').setAttribute('value',url);
  document.getElementById('delicious-post-dialog-description').setAttribute('value',description);
}


function delicious_post_dialog_accept() {
  delicious_postDialog();
	
  // general stuff
  var bundle=document.getElementById('delicious-post-stringbundle');
	
  // get values entered in the form
  var username=encodeURIComponent(document.getElementById('delicious-post-dialog-username').value);
  var password=encodeURIComponent(document.getElementById('delicious-post-dialog-password').value);

  // minimal sanity check
  if ((username=='')||(password=='')) {
    delicious_resetDialog(1);
    alertEx("", bundle.getString('supplylogin'));
    return false;
  }

  var url=encodeURIComponent(document.getElementById('delicious-post-dialog-url').value);
  var description=encodeURIComponent(document.getElementById('delicious-post-dialog-description').value);
  var extended=encodeURIComponent(document.getElementById('delicious-post-dialog-extended').value);
  var tags=encodeURIComponent(document.getElementById('delicious-post-dialog-tags').value);

  // handle the login saving stuff first
  const preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
  preferencesService.setCharPref('extensions.aecreations.deliciouspost.username',username);
  if (document.getElementById('delicious-post-dialog-use_password_manager').checked==true) {

    // Save new login
    if (oldpswd == "") {
      delicious_savePassword(username,password,true);
    }
    // Modify existing login
    if (oldpswd && password != oldpswd) {
      delicious_modifyPassword(username, oldpswd, password);
    }

    preferencesService.setBoolPref('extensions.aecreations.deliciouspost.use_password_manager',true);
  } else {
    // Remove saved password
    delicious_savePassword(username,'',false);
    preferencesService.setBoolPref('extensions.aecreations.deliciouspost.use_password_manager',false);
  }

  // work out the correct ISO datestamp (no need for timezone offset, we're using UTC throughout)
  var d = new Date();
  var isodatestamp = delicious_post_pad0(d.getUTCFullYear(),4)+'-'+delicious_post_pad0((d.getUTCMonth()+1),2)+'-'+delicious_post_pad0(d.getUTCDate(),2)+'T'+delicious_post_pad0(d.getUTCHours(),2)+':'+delicious_post_pad0(d.getUTCMinutes(),2)+':'+delicious_post_pad0(d.getUTCSeconds(),2)+'Z';
  // clean up - destroy created instance (necessary?)
  delete d;
		
  // build the entire query string
  var querystring = DELICIOUS_API_URL+'posts/add?url='+url+'&description='+description+'&extended='+extended+'&tags='+tags+'&dt='+isodatestamp;
		
  // start a new XMLHttpRequest 
  var xmlhttp = new XMLHttpRequest();
	
  // open the connection as synchronous, sending username/password as part of the HTTPAuth
  xmlhttp.open('GET',querystring,false,username,password);
	
  // send the actual request
  try {
    xmlhttp.send(null);
  } catch(e) {
    delicious_resetDialog(0);
    alertEx("", bundle.getString('statusconnectionerror'));
    return false;
  }
  try {
    switch(xmlhttp.status) {
    case 503:
      delicious_resetDialog(0);
      alertEx("", bundle.getString('statusthrottled'));
      return false;
    case 401:
    case 403:
      delicious_resetDialog(1);
      alertEx("", bundle.getString('statusauthenticationfailed'));
      return false;
    case 200:
      delicious_resetDialog(0);
      return true;
    default:
      delicious_resetDialog(0);
      return false;
    }
  } catch (e) {
    delicious_resetDialog(0);
    alertEx("", bundle.getString('statusconnectionerror'));
    return false;
  }

}


function delicious_autofillPassword(username) {
  if (document.getElementById('delicious-post-dialog-use_password_manager').checked==true) {
    var password = delicious_getPassword(username);
    document.getElementById('delicious-post-dialog-password').setAttribute('value',password);
  }
}

// helper function to pad numbers to required length (http://www.propix.hu/www/Clock/Clock.html)
function delicious_post_pad0(string, newlength) {
  var pad = '';
  var len = newlength-String(string).length;
  var i;
  for (i = 0; i<len; i++) {
    pad += '0';
  }
  return pad+string;
}


function delicious_savePassword(username,password,save) {
  var passwordManager;

  // Firefox 2
  if ("@mozilla.org/passwordmanager;1" in Components.classes) {
    passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance();

    if (passwordManager) {
      passwordManager = passwordManager.QueryInterface(Components.interfaces.nsIPasswordManager);
      if (save) {
	try {
	  passwordManager.addUser(DELICIOUSPOST_URL, username, password);
	} catch (e) { }
      } else {
	try {
	  passwordManager.removeUser(DELICIOUSPOST_URL, username);
	} catch (e) { }
      }	
    }
  }
  // Firefox 3 or higher
  else if ("@mozilla.org/login-manager;1" in Components.classes) {
    passwordManager = Components.classes["@mozilla.org/login-manager;1"]
      .getService(Components.interfaces
		  .nsILoginManager);
    if (save) {
      var newLogin = new nsLoginInfo(DELICIOUSPOST_URL, null, DELICIOUSPOST_HTTP_REALM, username, password, "", "");

      try {
	passwordManager.addLogin(newLogin);
      } catch (e) {}
    }
    else {
      var formSubmitURL = null;
      var logins;

      try {
	// Remove login info
	logins = passwordManager.findLogins({}, DELICIOUSPOST_URL, formSubmitURL, DELICIOUSPOST_HTTP_REALM);

	for (var i = 0; i < logins.length; i++) {
	  if (logins[i].username == username) {
	    try {
	      passwordManager.removeLogin(logins[i]);
	    } catch (e) {}
	    break;
	  }
	}
      } catch (e) {}
    }
  }
}


// NEW
function delicious_modifyPassword(username, oldPassword, newPassword)
{
  var passwordManager;

  // Firefox 3 or higher
  if ("@mozilla.org/login-manager;1" in Components.classes) {
    passwordManager = Components.classes["@mozilla.org/login-manager;1"]
      .getService(Components.interfaces
		  .nsILoginManager);
    var logins = passwordManager.findLogins({}, DELICIOUSPOST_URL, null, DELICIOUSPOST_HTTP_REALM);
    var newLogin = new nsLoginInfo(DELICIOUSPOST_URL, null, DELICIOUSPOST_HTTP_REALM, username, newPassword, "", "");
    var oldLogin;

    for (var i = 0; i < logins.length; i++) {
      if (logins[i].username == username) {
	oldLogin = logins[i];
      }
      break;
    }

    if (oldLogin) {
      passwordManager.modifyLogin(oldLogin, newLogin);
    }
  }
}


function delicious_getPassword(username) {
  var rv;
  var passwordManager, host, user, password;

  // Firefox 2
  if ("@mozilla.org/passwordmanager;1" in Components.classes) {
    passwordManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance(Components.interfaces.nsIPasswordManagerInternal);

    host = {value:''};
    user =  {value:''};
    password = {value:''}; 
    
    try {
      passwordManager.findPasswordEntry(DELICIOUSPOST_URL, username, '', host, user, password);
    } catch(e){ }

    rv = password.value;
  }
  // Firefox 3 or higher
  else if ("@mozilla.org/login-manager;1" in Components.classes) {
    passwordManager = Components.classes["@mozilla.org/login-manager;1"]
      .getService(Components.interfaces
		  .nsILoginManager);
    var formSubmitURL = null;
    var logins = passwordManager.findLogins({}, DELICIOUSPOST_URL, formSubmitURL, DELICIOUSPOST_HTTP_REALM);

    for (var i = 0; i < logins.length; i++) {
      if (logins[i].username == username) {
	password = logins[i].password;
      }
    }
  }

  if (password) {
    rv = decodeURIComponent(password);
  }

  return rv;
}


function delicious_postDialog() {	
  // temporarily disable accept/cancel buttons
  var dlg = document.getElementById('delicious-post-dialog');
  dlg.setAttribute('wait-cursor', true);
  dlg.getButton('accept').setAttribute('disabled', true);
}


function delicious_resetDialog(index) {
  var dlg = document.getElementById('delicious-post-dialog');
  dlg.removeAttribute('wait-cursor');
  dlg.getButton('accept').setAttribute('disabled', false);
  document.getElementById('delicious-post-dialog-tabbox').setAttribute('selectedIndex', index);
}



